/*!
 * グラデーション設定の取得.jsx
 * レイヤースタイルのグラデーションオーバーレイの設定を取得し、
 * linear-gradient()シンタックス値に整形して表示するスクリプト。
 * 
 * Version: 0.3.2(2012-01-21)
 * Author: Hideki Abe
 * 
 */

// ------------------------------------------------------------------------------
// グラデーション設定の取得
//   Note :
//   Adobe Forumの“How to get the style of a layer using photoshop scripting ??”
//   に掲載されたサンプルコードを元にしたもの。
//   see also: http://forums.adobe.com/message/3112948#3129254
//   
//   2012.01.17 Gradient Descriptorの取得処理を変更。
//              グラデーションオーバーレイのDescriptor取得処理を
//              getDescriptor関数へ移動し、角度・カラー情報が得られるようにした。
// ------------------------------------------------------------------------------
function getDescriptor() {
    try {
        var ref = new ActionReference(),
            descriptor;

        ref.putEnumerated(charIDToTypeID("Lyr "), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));
        descriptor = executeActionGet(ref).getObjectValue(stringIDToTypeID('layerEffects')).getObjectValue(stringIDToTypeID('gradientFill'));

        return descriptor;
    } catch(e) {
        return false;
    }
}

function getGradientAngle(descriptor) {
    var angle;

    angle = descriptor.getDouble(stringIDToTypeID('angle'));
    return angle;
}

function getGradientColors(descriptor) {
    var ref = new ActionReference();
    ref.putEnumerated(charIDToTypeID("Lyr "), charIDToTypeID("Ordn"), charIDToTypeID("Trgt"));

    // get gradient descriptor and gradient angle
    var desc = descriptor.getObjectValue(stringIDToTypeID('gradient'));

    // get an ActionList of color settings used in gradient
    var colorsList = desc.getList(stringIDToTypeID('colors'));
    var count = colorsList.count;// the number of color stops in gradient
    var colors = [];// array to hold custom color objects
    for( var c = 0; c < count; c++ ){
        desc = colorsList.getObjectValue(c);// get color descriptor
        var colorStop = {};// object to hold color stop info
        colorStop.location = desc.getInteger(stringIDToTypeID('location'));
        colorStop.midpoint = desc.getInteger(stringIDToTypeID('midpoint'));
        colorStop.type = typeIDToStringID(desc.getEnumerationValue(stringIDToTypeID('type')));

        var colorDesc = desc.getObjectValue(stringIDToTypeID('color'));
        var color = new SolidColor();
        switch(app.activeDocument.mode){
            case DocumentMode.GRAYSCALE:
                color.gray.gray = colorDesc.getInteger(charIDToTypeID('Gry '));
                break;
            case DocumentMode.RGB:
                color.rgb.red = colorDesc.getInteger(charIDToTypeID('Rd  '));
                color.rgb.green = colorDesc.getInteger(charIDToTypeID('Grn '));
                color.rgb.blue = colorDesc.getInteger(charIDToTypeID('Bl  '));
                break;
            case DocumentMode.CMYK:
                color.cmyk.cyan = colorDesc.getInteger(charIDToTypeID('Cyn '));
                color.cmyk.magenta = colorDesc.getInteger(charIDToTypeID('Mgnt'));
                color.cmyk.yellow = colorDesc.getInteger(charIDToTypeID('Ylw '));
                color.cmyk.black = colorDesc.getInteger(charIDToTypeID('Blck'));
                break;
            case DocumentMode.LAB:
                color.lab.l = colorDesc.getInteger(charIDToTypeID('Lmnc'));
                color.lab.a = colorDesc.getInteger(charIDToTypeID('A   '));
                color.lab.b = colorDesc.getInteger(charIDToTypeID('B   '));
                break;
        }
        colorStop.color = color;
        colors.push(colorStop);
    }
    return colors;
}

// ------------------------------------------------------------------------------
// 16進数変換
// ------------------------------------------------------------------------------
function conv16n(n) {
    var ret = n.toString(16);

    if(ret.length === 1) {
        ret = '0' + ret;
    }
    return ret;
}

// ------------------------------------------------------------------------------
// 情報表示処理
// ------------------------------------------------------------------------------
function action() {
    var descriptor,
        point,
        angle,
        deg = "",
        sideOrCorner = "",
        sideOrCornerOld = "",
        sideOrCornerSass = "",
        colorStops,
        colorStopsVal = "",
        colorStopsValSass = "",
        propval = "",
        dialog;

    // ディスクリプター取得
    descriptor = getDescriptor();
    if (!descriptor) {
        alert("グラデーションオーバーレイの設定がありません。");
        return false;
    }

    // 角度を方位角に変換・単位付与
    angle = getGradientAngle(descriptor);
    if (angle === -90) {
        deg = "";
    } else if (90 >= angle && angle >= -179) {
        deg = 90 - (angle) + "deg, ";
    } else if (90 < angle && angle <= 180) {
        deg = 270 + (180 - angle) + "deg, ";
    }

    // 0/90/180/270degはキーワードに変換
    // 直観的に判断できる方位角で分岐処理している
    // 上から下へのグラデーションはキーワード省略
    if (deg === "") {    // 180deg
        sideOrCorner = "";
        sideOrCornerOld = "";
    } else if (deg === "0deg, ") {
        sideOrCorner = "to top, ";
        sideOrCornerOld = "bottom, ";
    } else if (deg === "90deg, ") {
        sideOrCorner = "to right, ";
        sideOrCornerOld = "left, ";
    } else if (deg === "270deg, ") {
        sideOrCorner = "to left, ";
        sideOrCornerOld = "right, ";
    }

    // カラー情報の生成
    colorStops = getGradientColors(descriptor);
    for(i = 0, nColorStops = colorStops.length; i < nColorStops; i++) {
        point = Math.round(colorStops[i].location / 4096 * 100);
        if (colorStopsVal != "") {
            colorStopsVal += ", ";
        }
        colorStopsVal += '#' + conv16n(colorStops[i].color.rgb.red) + 
                         conv16n(colorStops[i].color.rgb.green) +
                         conv16n(colorStops[i].color.rgb.blue) +
                         ' ' + point + '%';
    }

    // linear-gradient()関数の出力
    propval = "[CSS]\r";
    if (sideOrCorner || angle === -90) {
        propval += "-(vendor-prefixes)-linear-gradient(" + sideOrCornerOld + colorStopsVal + ");\r" + 
                   "linear-gradient(" + sideOrCorner + colorStopsVal + ");\r" + 
                   "\r";
    } else {
        propval += "-(vendor-prefixes)-linear-gradient(" + angle + "deg, " + colorStopsVal + ");\r" + 
                   "linear-gradient(" + deg + colorStopsVal + ");\r" + 
                   "\r";
    }

    // Sass @includeの出力
    propval += "[Sass]\r";
    colorStopsValSass = colorStopsVal.replace(/#([0-9a-z]+)/g, "\"$1\"").replace(/\s(\d+)%/g, ", $1");
    if (sideOrCorner) {
        sideOrCornerSass = sideOrCorner.replace(/^(.*), $/, "\"$1\", ");
        propval += "@include linear-gradient(" + sideOrCornerSass + colorStopsValSass + ");\r";
    } else if (angle === -90) {
        propval += "@include linear-gradient(" + "\"to bottom\", " + colorStopsValSass + ");\r";
    } else {
        propval += "@include linear-gradient(" + deg + colorStopsValSass + ");\r";
    }

    // ダイアログ表示
    dialog = new Window('dialog', 'グラデーション設定情報');
    dialog.eText = dialog.add("edittext", [10, 10, 450, 10+150], propval, { multiline: true, scrollable: true });
    dialog.okBtn = dialog.add("button", [135, 150, 235, 150+25], "OK", { name: "ok" });
    dialog.show();
}

action();
